@@ -46,12 +46,12 @@ inst_store_by_set = {
         'lui     rd imm20 6..2=0x0D 1..0=3',
         'auipc   rd imm20 6..2=0x05 1..0=3',
         'addi    rd rs1 imm12           14..12=0 6..2=0x04 1..0=3',
-        'slli    rd rs1 31..26=0  shamt 14..12=1 6..2=0x04 1..0=3',
         'slti    rd rs1 imm12           14..12=2 6..2=0x04 1..0=3',
         'sltiu   rd rs1 imm12           14..12=3 6..2=0x04 1..0=3',
         'xori    rd rs1 imm12           14..12=4 6..2=0x04 1..0=3',
-        'srli    rd rs1 31..26=0  shamt 14..12=5 6..2=0x04 1..0=3',
-        'srai    rd rs1 31..26=16 shamt 14..12=5 6..2=0x04 1..0=3',
+        'slli    rd rs1 31..25=0  shamtw 14..12=1 6..2=0x04 1..0=3',
+        'srli    rd rs1 31..25=0  shamtw 14..12=5 6..2=0x04 1..0=3',
+        'srai    rd rs1 31..25=32 shamtw 14..12=5 6..2=0x04 1..0=3',
         'ori     rd rs1 imm12           14..12=6 6..2=0x04 1..0=3',
         'andi    rd rs1 imm12           14..12=7 6..2=0x04 1..0=3',
         'add     rd rs1 rs2 31..25=0  14..12=0 6..2=0x0C 1..0=3',
@@ -89,6 +89,9 @@ inst_store_by_set = {
         'sllw    rd rs1 rs2 31..25=0  14..12=1 6..2=0x0E 1..0=3',
         'srlw    rd rs1 rs2 31..25=0  14..12=5 6..2=0x0E 1..0=3',
         'sraw    rd rs1 rs2 31..25=32 14..12=5 6..2=0x0E 1..0=3',
+        'slli    rd rs1 31..26=0  shamt 14..12=1 6..2=0x04 1..0=3',
+        'srli    rd rs1 31..26=0  shamt 14..12=5 6..2=0x04 1..0=3',
+        'srai    rd rs1 31..26=16 shamt 14..12=5 6..2=0x04 1..0=3',
     ],
     'rv64i.data' : [
         'ld      rd rs1       imm12 14..12=3 6..2=0x00 1..0=3',
@@ -247,7 +250,6 @@ inst_store_by_set = {
         'c.xor rd_rs1_prime rsprime2',
         'c.or rd_rs1_prime rsprime2',
         'c.and rd_rs1_prime rsprime2',
-        'c.slli rd_rs1_prime nzuimm6',
         'c.add rd_rs1 rs2',
         'c.mv rd x0 rs2',
     ],
@@ -284,6 +286,7 @@ inst_store_by_set = {
         'c.addiw rd_rs1_prime nzimm6',
         'c.srli rd_rs1_prime nzuimm6',
         'c.srai rd_rs1_prime nzuimm6',
+        'c.slli rd_rs1_prime nzuimm6',
         'c.subw rd_rs1_prime rsprime2',
         'c.addw rd_rs1_prime rsprime2',
     ],
